######################################################################
##  This code includes the calculation of the REST statistic, 
##  performing the heterogeneity test.
##  Details of the procedures can be found in: 
##      Thompson, K.L. and S. Lin.  A robust and efficient
##          statistic for detecting heterogeneous cancer samples.
######################################################################



#################################################################
## DEFINITION OF REST FUNCTIONS
#################################################################

## Function to calculate the REST statistic for each gene

rest.stat.gene<-function(B,labels,nu,c,k=FALSE)
	{
    #B: vector of gene expressions for n samples
    #labels: logical vector of sample labels (TRUE=disease, FALSE=normal)
    #nu: power of nu in definition of proposed statistic (default=1)
    #c: penalty for normal sample appearing in proposed statistic's path (default=1/n1)
    #k: flag (TRUE indicates to save k.hat and mu.hat)
  
  labels<-labels[!is.nan(B)]
  B<-B[!is.nan(B)]
  
  A<-cbind(labels,B)
  sorted.expr<-A[order(A[,2],decreasing=TRUE),]
  sorted.expr1<-apply(sorted.expr,1,prod) #imputs zeroes for expression values for normal samples
  our.stat.path<- abs( cumsum(c*(sorted.expr1==0) + abs(sorted.expr1)^nu/sum(abs(sorted.expr1)^nu)))
  k.hat=sum(sorted.expr[1:match(max(our.stat.path),our.stat.path),1])
  max.loc=match(max(our.stat.path),our.stat.path)
  if(k) return(c(max(our.stat.path),k.hat,mean(sorted.expr[sorted.expr[,1]==1,2][1:k.hat]),max.loc))
  else return(max(our.stat.path))
  
	}

## Function to calculate REST statistic on a gene expression matrix

reststat <- function (expr,labels,nu=1,c=-1/sum(!labels),k=FALSE,perm=FALSE,nreps=1000)
{
	#Function returns the calculated value of the REST statistic and a p-value for
	#	the permutation test (if perm=TRUE) in a matrix with two rows

	#expr: matrix of gene expressions with rows being samples, columns being genes
	#labels: logical vector of sample labels (TRUE=disease, FALSE=normal)
	#nu: power of nu in definition of proposed statistic (default=1)
	#c: penalty for normal sample appearing in proposed statistic's path (default=1/n1)
	#perm: logical variable (TRUE=perform permutation testing, FALSE=no permutation testing)
	#nreps: number of permutations to use (meaningless if perm=FALSE)
	
	if(!perm)
	{
		rest.stat<-apply(as.matrix(expr),2,rest.stat.gene,nu=nu,c=c,labels=labels,k=k)
		return(rest.stat)
	}
	else if(perm) {

		rest.stat<-apply(as.matrix(expr),2,rest.stat.gene,nu=nu,c=c,labels=labels,k=k)
		expr.perm<-matrix(NaN,nrow=dim(expr)[1],ncol=nreps)
		p.val.rest<-rep(NA, dim(expr)[2])
		dummy<-is.na(expr)

		for (h in 1:dim(expr)[2])
		{
			values<-(1:dim(expr)[1])[!dummy[,h]]
			for (reps in 1:nreps)	expr.perm[values,reps]<-expr[sample(values),h]

			rest.stat.perm<-apply(as.matrix(expr.perm),2,rest.stat.gene,nu=nu,c=c,labels=labels,k=k)
			p.val.rest[h]<-mean(rest.stat[h]<=rest.stat.perm)             
		}

		return(rbind(rest.stat,p.val.rest))
	}
}


## Function to perform heterogeneity test
heterogeneitytest <- function(expr,labels,test.true=1:dim(expr)[2],alpha=0.05,nreps=1000)
{
	# Function returns a vector of the indices of genes for which 
	#	the test rejected the null hypothesis of the heterogeneity test
	
	#expr: matrix of gene expressions with rows being samples, columns being genes
	#labels: logical vector of sample labels (TRUE=disease, FALSE=normal)
	#test.true: vector of integers indicating the genes where hypothesis test is needed
	#nreps: number of null data sets simulate for null distribution
	#alpha: level of heterogeneity test


	##Calculating Heterogeneity Test Statistic
	A=cbind(labels,expr)
	kmu<-reststat(expr,labels,k=TRUE)[2:3,]
	k.hat<-kmu[1,]
	mu2.hat.mat<-kmu[2,]
	mu2.hat.mat[is.na(mu2.hat.mat)]<-0
	n2.vec<-n1.vec<-rep(NA,dim(expr)[2])

	##Calculating Sample Size for each Gene
	for(col in 2:(dim(expr)[2]+1))
	{
	  B=A[order(A[,col],decreasing=TRUE),c(1,col)]
	  B=B[!is.nan(B[,2]),]
	  n2.vec[col-1]=sum(B[,1])
	  n1.vec[col-1]=sum(1-B[,1])
	}
	
	T.pval<-rep(NA,dim(expr)[2])

	for (h in test.true)
	{
	  mu2.hat<-mu2.hat.mat[h]
	  n1<-n1.vec[h]
	  n2<-n2.vec[h]
	  
	  #Simulating Null Distribution
	  x=matrix(rnorm(nreps*n1,mean=0,sd=1),nrow=n1) #Simulate normal data
	  y=matrix(rnorm(nreps*n2,mean=mu2.hat,sd=1),nrow=n2) #Simulate disease data
	  permmat=rbind(x,y)
	  labels=c(rep(FALSE,n1),rep(TRUE,n2))
	  
	  k.hat.dist<-reststat(permmat,labels,k=TRUE)[2,]
	  T.pval[h]<-mean(k.hat[h]>=k.hat.dist)
	  
	}

	#Make decision for test
	test.reject=which(T.pval<=alpha)            
	test.reject=test.reject[!is.na(test.reject)]

	return(test.reject)
}